import _Object$values from 'babel-runtime/core-js/object/values';
import _Object$keys from 'babel-runtime/core-js/object/keys';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import { FormItemModel } from './FormItemModel';

var FormsModel = function FormsModel() {
  var _this = this;

  _classCallCheck(this, FormsModel);

  this.get = function (name, defaultValue) {
    return _this.forms[name] || defaultValue;
  };

  this.add = function (_ref, asList) {
    var name = _ref.name,
        validator = _ref.validator,
        excluded = _ref.excluded;

    var model = new FormItemModel(name, validator, excluded);
    if (asList) {
      if (!_this.forms[name]) {
        _this.forms[name] = [];
      }
      _this.forms[name].push(model);
    } else {
      _this.forms[name] = model;
    }
    return model;
  };

  this.remove = function (model) {
    var name = model.name;
    if (Array.isArray(_this.forms[name])) {
      var index = _this.forms[name].findIndex(function (form) {
        return form.id === model.id;
      });
      if (index > -1) {
        _this.forms[name].splice(index, 1);
      }
      if (!_this.forms[name].length) {
        delete _this.forms[name];
      }
    } else {
      delete _this.forms[name];
    }
  };

  this.validate = function () {
    _this.list().forEach(function (form) {
      return form.validate();
    });
  };

  this.reload = function () {
    _this.list().forEach(function (form) {
      return form.reload();
    });
  };

  this.addErrors = function (errors) {
    _Object$keys(_this.forms).forEach(function (key) {
      if (Array.isArray(_this.forms[key])) {
        var formsErrors = errors[key] || [];
        return _this.forms[key].forEach(function (form, index) {
          return form.addErrors(formsErrors[index]);
        });
      }
      return _this.forms[key].addErrors(errors[key]);
    });
  };

  this.init = function (data) {
    _Object$keys(_this.forms).forEach(function (key) {
      if (Array.isArray(_this.forms[key])) {
        var formsValues = data[key] || [];
        return _this.forms[key].forEach(function (form, index) {
          return form.init(formsValues[index]);
        });
      }
      return _this.forms[key].init(data[key]);
    });
  };

  this.reset = function () {
    _this.list().forEach(function (form) {
      return form.reset();
    });
  };

  this.makeDirty = function () {
    _this.list().forEach(function (form) {
      return form.makeDirty();
    });
  };

  this.makeSubmitted = function () {
    _this.list().forEach(function (form) {
      return form.makeSubmitted();
    });
  };

  this.isValid = function () {
    return _this.list().every(function (form) {
      return form.isValid();
    });
  };

  this.isTouched = function () {
    return _this.list().some(function (form) {
      return form.isTouched();
    });
  };

  this.data = function () {
    var values = {};
    _Object$keys(_this.forms).reduce(function (prev, key) {
      if (Array.isArray(_this.forms[key])) {
        prev[key] = _this.forms[key].map(function (form) {
          return form.data();
        });
        if (!prev[key].length) {
          delete prev[key];
        }
      } else {
        prev[key] = _this.forms[key].data();
      }
      return prev;
    }, values);
    return values;
  };

  this.values = function () {
    var values = {};
    _Object$keys(_this.forms).reduce(function (prev, key) {
      if (Array.isArray(_this.forms[key])) {
        prev[key] = _this.forms[key].filter(function (form) {
          return !form.excluded;
        }).map(function (form) {
          return form.values();
        });
        if (!prev[key].length) {
          delete prev[key];
        }
      } else if (!_this.forms[key].excluded) {
        prev[key] = _this.forms[key].values();
      }
      return prev;
    }, values);
    return values;
  };

  this.errors = function () {
    var errors = {};
    _Object$keys(_this.forms).forEach(function (key) {
      if (Array.isArray(_this.forms[key])) {
        errors[key] = [];
        return _this.forms[key].forEach(function (form) {
          errors[key].push(form.getErrors());
        });
      }
      errors[key] = _this.forms[key].getErrors();
    });
    return errors;
  };

  this.list = function () {
    return _Object$values(_this.forms).reduce(function (list, current) {
      if (Array.isArray(current)) {
        list = [].concat(list, current);
      } else {
        list.push(current);
      }
      return list;
    }, []);
  };

  this.getPublic = function () {
    var forms = {};
    _Object$keys(_this.forms).forEach(function (key) {
      if (Array.isArray(_this.forms[key])) {
        forms[key] = [];
        return _this.forms[key].forEach(function (form) {
          forms[key].push(form.getPublic());
        });
      }
      forms[key] = _this.forms[key].getPublic();
    });
    return forms;
  };

  this.forms = {};
};

export default FormsModel;